@@ -14,6 +14,7 @@
 
 from six.moves.urllib import parse
 
+from zunclient import api_versions
 from zunclient.common import base
 from zunclient.common import utils
 from zunclient import exceptions
@@ -24,7 +25,8 @@ CREATION_ATTRIBUTES = ['name', 'image', 'command', 'cpu', 'memory',
                        'restart_policy', 'interactive', 'image_driver',
                        'security_groups', 'hints', 'nets', 'auto_remove',
                        'runtime', 'hostname', 'mounts', 'disk',
-                       'availability_zone', 'auto_heal']
+                       'availability_zone', 'auto_heal', 'privileged',
+                       'exposed_ports', 'healthcheck']
 
 
 class Container(base.Resource):
@@ -44,7 +46,7 @@ class ContainerManager(base.Manager):
             return '/v1/containers'
 
     def list(self, marker=None, limit=None, sort_key=None,
-             sort_dir=None, detail=False, all_projects=False, **kwargs):
+             sort_dir=None, all_projects=False, **kwargs):
         """Retrieve a list of containers.
 
         :param all_projects: Optional, list containers in all projects
@@ -56,8 +58,7 @@ class ContainerManager(base.Manager):
                       request, if:
 
             1) limit > 0, the maximum number of containers to return.
-            2) limit == 0, return the entire list of containers.
-            3) limit param is NOT specified (None), the number of items
+            2) limit param is NOT specified (None), the number of items
                returned respect the maximum imposed by the ZUN API
                (see Zun's api.max_limit option).
 
@@ -66,9 +67,6 @@ class ContainerManager(base.Manager):
         :param sort_dir: Optional, direction of sorting, either 'asc' (the
                          default) or 'desc'.
 
-        :param detail: Optional, boolean whether to return detailed information
-                       about containers.
-
         :returns: A list of containers.
 
         """
@@ -78,8 +76,6 @@ class ContainerManager(base.Manager):
         filters = utils.common_filters(marker, limit, sort_key,
                                        sort_dir, all_projects)
         path = ''
-        if detail:
-            path += 'detail'
         if filters:
             path += '?' + '&'.join(filters)
 
@@ -99,6 +95,8 @@ class ContainerManager(base.Manager):
             return None
 
     def create(self, **kwargs):
+        self._process_command(kwargs)
+
         new = {}
         for (key, value) in kwargs.items():
             if key in CREATION_ATTRIBUTES:
@@ -108,6 +106,13 @@ class ContainerManager(base.Manager):
                     "Key must be in %s" % ','.join(CREATION_ATTRIBUTES))
         return self._create(self._path(), new)
 
+    def _process_command(self, kwargs):
+        cmd_microversion = api_versions.APIVersion("1.20")
+        if self.api_version < cmd_microversion:
+            command = kwargs.pop('command', None)
+            if command:
+                kwargs['command'] = utils.parse_command(command)
+
     def delete(self, id, **kwargs):
         return self._delete(self._path(id),
                             qparams=kwargs)
@@ -164,6 +169,8 @@ class ContainerManager(base.Manager):
                             qparams={'signal': signal})[1]
 
     def run(self, **kwargs):
+        self._process_command(kwargs)
+
         if not set(kwargs).issubset(CREATION_ATTRIBUTES):
             raise exceptions.InvalidAttribute(
                 "Key must be in %s" % ','.join(CREATION_ATTRIBUTES))
@@ -189,10 +196,19 @@ class ContainerManager(base.Manager):
                             qparams={'ps_args': ps_args})[1]
 
     def get_archive(self, id, path):
-        return self._action(id, '/get_archive', method='GET',
-                            qparams={'path': path})[1]
+        res = self._action(id, '/get_archive', method='GET',
+                           qparams={'path': path})[1]
+        # API version 1.25 or later will return Base64-encoded data
+        if self.api_version >= api_versions.APIVersion("1.25"):
+            res['data'] = utils.decode_file_data(res['data'])
+        else:
+            res['data'] = res['data'].encode()
+        return res
 
     def put_archive(self, id, path, data):
+        # API version 1.25 or later will expect Base64-encoded data
+        if self.api_version >= api_versions.APIVersion("1.25"):
+            data = utils.encode_file_data(data)
         return self._action(id, '/put_archive',
                             qparams={'path': path},
                             body={'data': data})
